#!/bin/sh

__qpod_var_reset() {
    args=
    container=
}

__qpod_intern_check_podman() {
    podman --version > /dev/null 2>&1
}

__qpod_intern_parse_args() {
    container="$1"
    # remove "container" from positional parameters
    shift 1
    # create argument list and quote each arbgument to ensure it's passed in the same form it's given in
    args=""
    for arg in $@; do
        if [ -z "${args}" ]; then
            args="\"${arg}\""
        else
            args="${args} \"${arg}\""
        fi
    done
}

__qpod_intern_oload_check_simple_option() {
    echo -n "$arg" | grep -E -- "^-[dfhlqvwAEFORSW]+" > /dev/null 2>&1 && return 0
    echo -n "$arg" | grep -E -- "^--(allow-redef|force|full-release|help)" > /dev/null 2>&1 && return 0
    echo -n "$arg" | grep -E -- "^--(no-error-update|override|quiet|reload)" > /dev/null 2>&1 && return 0
    echo -n "$arg" | grep -E -- "^--(show-(defines|full-error|url))" > /dev/null 2>&1 && return 0
    echo -n "$arg" | grep -E -- "^--(use-schedule|verbose|validate)" > /dev/null 2>&1 && return 0
    echo -n "$arg" | grep -E -- "^--(import-cfg-val-rest|sign-schema)" > /dev/null 2>&1 && return 0
    return 1
}

__qpod_intern_oload_check_long_param_option() {
    echo -n "$arg" | grep -E -- "^--(user-)?schema=" > /dev/null 2>&1 && return 0
    echo -n "$arg" | grep -E -- "^--(proxy-)?url=" > /dev/null 2>&1 && return 0
    echo -n "$arg" | grep -E -- "^--(data|index)-ts=" > /dev/null 2>&1 && return 0
    echo -n "$arg" | grep -E -- "^--delete(-id)?=" > /dev/null 2>&1 && return 0
    echo -n "$arg" | grep -E -- "^--(datasource|list|refresh|token)=" > /dev/null 2>&1 && return 0
    echo -n "$arg" | grep -E -- "^--(export-cfg-val|show-release)" > /dev/null 2>&1 && return 0
    return 1
}

__qpod_intern_oload_check_short_param_option() {
    echo -n "$arg" | grep -E -- "^-[prstuDLX]$" > /dev/null 2>&1 && return 0
    return 1
}

__qpod_intern_oload_check_short_param_option_with_arg() {
    echo -n "$arg" | grep -E -- "^-[prstuDLX]" > /dev/null 2>&1 && return 0
    return 1
}

__qpod_intern_oload_parse_args() {
    oload_files=
    oload_opts=
    for arg in $@; do
        # parameter of short option
        if [ "$next_arg_append_opts" = "1" ]; then
            oload_opts="${oload_opts} ${arg}"
            next_arg_append_opts=
            continue
        fi

        # if simple option, add to args and skip
        if __qpod_intern_oload_check_simple_option; then
            oload_opts="${oload_opts} ${arg}"
            continue
        fi

        # if long param option, add to args and skip
        if __qpod_intern_oload_check_long_param_option; then
            oload_opts="${oload_opts} ${arg}"
            continue
        fi

        # if short param option + next arg param, add to oload opts and set helper var for the next arg
        if __qpod_intern_oload_check_short_param_option; then
            oload_opts="${oload_opts} ${arg}"
            next_arg_append_opts=1
            continue
        fi

        if __qpod_intern_oload_check_short_param_option_with_arg; then
            oload_opts="${oload_opts} ${arg}"
            continue
        fi

        # else file -> add to list
        oload_files="${oload_files} ${arg}"
    done
}

__qpod_intern_oload_check_yaml_files() {
    for ofile in ${oload_files}; do
        if echo -n ${ofile} | grep -E "\.y[a]?ml" > /dev/null 2>&1; then
            srcfile=`echo -n $ofile | sed 's/\.yml//' | sed 's/\.yaml//'`
            if [ -e "$srcfile" ]; then
                oload_files="${oload_files} ${srcfile}"
            fi
        fi
    done
    oload_files=`echo $oload_files | tr ' ' '\n' | sort -u | xargs`
}

__qpod_intern_oload_check_resource_files() {
    resource_files=
    for ofile in ${oload_files}; do
        # do not process oload args
        if case ${ofile} in -*) ;; *) false;; esac; then
            continue
        fi
        file_dir=`dirname $ofile`
        echo "Checking resources for file '${ofile}'"

        # parse yaml file
        if echo -n ${ofile} | grep -E "\.y[a]?ml" > /dev/null 2>&1; then
            # skip if the file doesn't contain a resource key
            if ! grep -E "^resource:" ${ofile} > /dev/null 2>&1; then
                continue
            fi

            # parse the file and it's resource section
            resources=
            multiline_item_list=
            multiline_bracket_list=
            while IFS= read -r line || [[ -n "$line" ]]; do
                if [ "$multiline_item_list" = "1" ]; then
                    # check if the line is a list item (begins with dash)
                    if echo -n ${line} | grep -E "^[[:blank:]]*-" > /dev/null 2>&1; then
                        rsrc=`echo -n $line | sed 's/^[[:blank:]]*-[[:blank:]]*//' | sed 's/"//g' | xargs`
                        resources="${resources} ${rsrc}"
                        continue
                    fi
                    break
                fi

                if [ "$multiline_bracket_list" = "1" ]; then
                    # check if there is a closing bracket in the line
                    if echo -n ${line} | grep -E "\]" > /dev/null 2>&1; then
                        # parse until closing bracket
                        rsrc=`echo -n $line | sed 's/[[:blank:]]*\(.*\)\].*/\1/g' | sed 's/,/ /g' | sed 's/"//g' | xargs`
                        resources="${resources} ${rsrc}"
                        break
                    fi

                    # parse line
                    rsrc=`echo -n $line | sed 's/,/ /g' | sed 's/"//g' | xargs`
                    resources="${resources} ${rsrc}"

                elif echo -n ${line} | grep -E "^resource:[[:blank:]]*\[" > /dev/null 2>&1; then
                    # parse the rest of the line after opening bracket
                    rsrc=`echo -n $line | sed 's/^resource:[[:blank:]]*\[//'`

                    # check if there is a closing bracket in the line
                    if echo -n ${rsrc} | grep -E "\]" > /dev/null 2>&1; then
                        # parse until closing bracket
                        rsrc=`echo -n $rsrc | sed 's/\].*//' | sed 's/,/ /g' | sed 's/"//g' | xargs`
                        resources="${resources} ${rsrc}"
                        break
                    fi

                    # we are in the middle of multi-line bracket list
                    multiline_bracket_list=1

                    # parse the resource string from the previously parsed line
                    rsrc=`echo -n $rsrc | sed 's/,/ /g' | sed 's/"//g' | xargs`
                    resources="${resources} ${rsrc}"

                elif echo -n ${line} | grep -E "^resource:[[:blank:]]*$" > /dev/null 2>&1; then
                    # this is a beginning of multiline list (dashes in front of items)
                    multiline_item_list=1
                fi
            done < "${ofile}"

        # parse old-style qsd file
        elif echo -n ${ofile} | grep -E "\.qsd" > /dev/null 2>&1; then
            # skip if the file doesn't contain resource metadata option
            if ! grep -E "#[ ]*resource:" ${ofile} > /dev/null 2>&1; then
                continue
            fi

            # get resource strings from the metadata options
            resources=`sed 's/#[ ]*resource:[ ]*/XXXOLOAD_RSRC:/' $ofile | grep "XXXOLOAD_RSRC:" | xargs | sed 's/XXXOLOAD_RSRC://g'`

        # non-yaml and non-service files
        else
            continue
        fi

        # get full relative paths of the resources
        resources_full=
        for rsrc in ${resources}; do
            resources_full="${resources_full} ${file_dir}/${rsrc}"
        done
        if [ "$resources_full" != "" ]; then
            resources_full=`ls $resources_full 2>/dev/null | xargs`
            echo "Found resources: $resources"
        fi

        # add to resource files list
        resource_files="${resource_files} ${resources_full}"
    done

    # clean up the resource file list
    resource_files=`echo $resource_files | tr ' ' '\n' | sort -u | xargs`
}

__qpod_intern_oload_mkdir() {
    podman exec -it ${container} mkdir -p $1
}

__qpod_intern_oload_mk_tmpdir() {
    __qpod_intern_oload_mkdir ${oload_tmpdir}
}

__qpod_intern_oload_rm_tmpdir() {
    podman exec -it ${container} rm -rf ${oload_tmpdir}
}

__qpod_intern_container_running() {
    podman ps --format "{{.Names}}" | grep "$container" > /dev/null 2>&1
}

__qpod_intern_check_container() {
    if [ "$container" == "" ]; then
        echo "Error: Qorus container not set." >&2
        return 1
    fi
    if ! __qpod_intern_container_running; then
        echo "Error: There is no running Qorus container named '${container}'." >&2
        return 1
    fi
}

__qpod_intern_check_args() {
    __qpod_intern_check_container
}

__qpod_intern_print_args() {
    echo "Executing ${cmd} in container: '${container}'"
    echo "Arguments: '${args}'" && echo "-----"
}

__qpod_intern_exec_docker_cmd() {
    podman exec -it ${container} bash -c ". /opt/qorus/bin/env.sh && ${cmd} ${args}"
}

__qpod_intern_do_cmd() {
    __qpod_var_reset
    __qpod_intern_check_podman && \
    __qpod_intern_parse_args $@ && \
    __qpod_intern_check_args && \
    __qpod_intern_print_args && \
    __qpod_intern_exec_docker_cmd
}

qpod-ocmd() {
    cmd=ocmd
    __qpod_intern_do_cmd $@
}

qpod-ojview() {
    cmd=ojview
    __qpod_intern_do_cmd $@
}

qpod-oload() {
    cmd=oload
    __qpod_var_reset
    __qpod_intern_check_podman || return 1
    __qpod_intern_parse_args $@ || return 1
    __qpod_intern_check_args || return 1
    __qpod_intern_print_args || return 1

    # parse args for options and loaded files
    oload_files=
    __qpod_intern_oload_parse_args ${args} || return 1

    # check for yaml files and add their source files if they are missing
    __qpod_intern_oload_check_yaml_files

    # check service files for resources
    __qpod_intern_oload_check_resource_files

    # create temp dir in container for copying oloaded files
    oload_tmpdir=/tmp/.qpod-oload-`head -c 192 /dev/urandom | tr -dc A-Za-z0-9 | head -c 24`
    if ! __qpod_intern_oload_mk_tmpdir; then
        echo "Error: Could not create temporary directory inside container '${container}'." >&2
        return 1
    fi

    # copy resource files to temp dir in container
    for rsrcfile in ${resource_files}; do
        echo "Copying resource file '${rsrcfile}' to container"

        # create matching directory structure for the file inside the temp dir
        file_dir=`dirname $rsrcfile`
        if [ "$file_dir" == "." ]; then
            file_dir=
        else
            if ! echo -n "$file_dir" | grep -E "^/" > /dev/null 2>&1; then
                file_dir=/${file_dir}
            fi
            __qpod_intern_oload_mkdir ${oload_tmpdir}${file_dir}
        fi

        # copy the file into the tempdir
        if ! podman cp -- ${rsrcfile} ${container}:${oload_tmpdir}${file_dir}/; then
            echo "Error: Could not copy resource file '${rsrcfile}' into container '${container}'." >&2
            __qpod_intern_oload_rm_tmpdir
            return 1
        fi
    done

    # copy loaded files to temp dir in container
    oload_files_final=
    for ofile in ${oload_files}; do
        # do not copy oload args
        if case ${ofile} in -*) ;; *) false;; esac; then
            # add arg
            oload_files_final="${oload_files_final} ${ofile}"
            continue
        fi
        echo "Copying file '${ofile}' to container"

        # create matching directory structure for the file inside the temp dir
        file_dir=`dirname $ofile`
        if [ "$file_dir" == "." ]; then
            file_dir=
        else
            if ! echo -n "$file_dir" | grep -E "^/" > /dev/null 2>&1; then
                file_dir=/${file_dir}
            fi
            __qpod_intern_oload_mkdir ${oload_tmpdir}${file_dir}
        fi

        # copy the file into the tempdir
        if ! podman cp -- ${ofile} ${container}:${oload_tmpdir}${file_dir}/; then
            echo "Error: Could not copy file '${ofile}' into container '${container}'." >&2
            __qpod_intern_oload_rm_tmpdir
            return 1
        fi
        oload_files_final="${oload_files_final} ${oload_tmpdir}${file_dir}/`basename $ofile`"
    done
    oload_files_final=`echo -n "${oload_files_final}" | xargs`

    echo "Executing ${cmd}..."
    echo "-----"

    # execute oload
    podman exec -it ${container} bash -c ". /opt/qorus/bin/env.sh && cd ${oload_tmpdir} && ${cmd} ${oload_opts} ${oload_files_final}"
    ret=$?

    # remove temp dir with files
    __qpod_intern_oload_rm_tmpdir
    return ${ret}
}

qpod-oprop() {
    cmd=oprop
    __qpod_intern_do_cmd $@
}

qpod-ostart() {
    cmd=ostart
    __qpod_intern_do_cmd $@
}

qpod-ostatus() {
    cmd=ostatus
    __qpod_intern_do_cmd $@
}

qpod-ostop() {
    cmd=ostop
    __qpod_intern_do_cmd $@
}

qpod-oview() {
    cmd=oview
    __qpod_intern_do_cmd $@
}

qpod-qctl() {
    cmd=qctl
    __qpod_intern_do_cmd $@
}

qpod-qdbg() {
    cmd=qdbg
    __qpod_intern_do_cmd $@
}

qpod-qdbg-remote() {
    cmd=qdbg-remote
    __qpod_intern_do_cmd $@
}

qpod-qrest() {
    cmd=qrest
    __qpod_intern_do_cmd $@
}

qpod-schema-tool() {
    cmd=schema-tool
    __qpod_intern_do_cmd $@
}

qpod-qbugreport() {
    cmd=qbugreport
    __qpod_intern_do_cmd $@
}

qpod-user-tool() {
    cmd=user-tool
    __qpod_intern_do_cmd $@

# qore commands
qpod-rest() {
    cmd=rest
    __qpod_intern_do_cmd $@
}

qpod-schema-reverse() {
    cmd=schema-reverse
    __qpod_intern_do_cmd $@
}

qpod-sfrest() {
    cmd=sfrest
    __qpod_intern_do_cmd $@
}

qpod-soaputil() {
    cmd=soaputil
    __qpod_intern_do_cmd $@
}

qpod-sqlutil() {
    cmd=sqlutil
    __qpod_intern_do_cmd $@
}

qpod-qdp() {
    cmd=qdp
    __qpod_intern_do_cmd $@
}

qpod-saprest() {
    cmd=saprest
    __qpod_intern_do_cmd $@
}

#make-release
#qbugreport
#qdbg
#qdbg-remote
#qdbg-server
#qevent
#qget
#qhaorder
#qorus-diff
