#!/bin/sh

__remote_var_reset() {
    args=
    argsarray=
    dir=
    file_list=
    machine=
    make_release_files=
    make_release_files_final=
    make_release_files_inter=
    make_release_label=
    make_release_opts=
    make_release_src_dir=
    netrc_file=
    nodelete=
    ofile_name=
    ofile_content=
    oload_files=
    oload_files_final=
    oload_opts=
    port=
    qfile_content=
    qfile_name=
    timeout=
    tmp_files=
    secure=
    separator=
    URL=
    verbose=
}

__reset_token() {
    has_token=
}

__remote_print() {
    if [ ! -z ${verbose} ] && [ ${verbose} == "yes" ]; then
        echo $1
    fi
}

__remote_print_usage() {
    echo "Usage: remote-${cmd} <NETRC-FILE> [<COMMAND-ARGS> ...]"
}

__extract_URL() {
    if [ ! -e "${netrc_file}" ]; then
        echo "netrc configuration file \"${netrc_file}\" does not exist"
        return 1
    fi

    verbose=$(cat "${netrc_file}" | grep "verbose")
    verbose=$(echo ${verbose} | sed 's/verbose //g')
    machine=$(cat "${netrc_file}" | grep "machine ")
    machine=$(echo ${machine} | sed 's/machine //g')
    port=$(cat "${netrc_file}" | grep "port")
    port=$(echo ${port} | sed 's/port //g')
    secure=$(cat "${netrc_file}" | grep "secure")
    secure=$(echo ${secure} | sed 's/secure //g')
    timeout=$(cat "${netrc_file}" | grep "timeout")
    timeout=$(echo ${timeout} | sed 's/timeout //g')
    if [ ! -z ${timeout} ]; then
        timeout="-m ${timeout}"
    fi
    nodelete=$(cat "${netrc_file}" | grep "nodelete")
    nodelete=$(echo ${nodelete} | sed 's/nodelete //g')

    if [ -z ${machine} ] && [ -z ${port} ] && [ -z ${secure} ]; then
        echo "Impossible to find the netrc configuration in this file: \"${netrc_file}\""
        return 1
    fi

    if [ -z ${machine} ]; then
        echo "\"machine\" field is not defined in the netrc configuration file \"${netrc_file}\""
        return 1
    elif [ -z ${port} ]; then
        echo "\"port\" field is not defined in the netrc configuration file \"${netrc_file}\""
        return 1
    elif [ -z ${secure} ]; then
        echo "\"secure\" field is not defined in the netrc configuration file \"${netrc_file}\""
        return 1
    fi

    if [ ${secure} ] && [ ${secure} == "yes" ]; then
        URL="https://${machine}:${port}/"
    else
        URL="http://${machine}:${port}/"
    fi
}

__remote_intern_parse_args() {
    if [ -z "$1" ] || [ "$1" == "-h" -o "$1" == "--help" -o "$1" == "--usage" ]; then
        __remote_print_usage
        return 1
    fi

    netrc_file="$1"
    if ! __extract_URL; then
        return 1
    fi

    # remove "netrc_file" from positional parameters
    shift 1

    # create argument list and quote each argument to ensure it's passed in the same form it's given in
    args=""
    for arg in "$@"; do
        if [ -z "${args}" ]; then
            args="\"${arg}\""
        else
            args="${args}${separator}\"${arg}\""
        fi
    done
}

__remote_intern_exec_cmd() {
    __remote_print "curl -k -sS ${timeout} --netrc-file ${netrc_file} ${URL}api/latest/command -d cmd=${cmd}${has_token}&args=${args}"
    ret=$(curl -k -sS ${timeout} --netrc-file ${netrc_file} ${URL}api/latest/command -d "cmd=${cmd}${has_token}&args=${args}")
    if [[ $? > 0 ]] || [[ "${ret}" =~ "^<html><head><title>" ]]; then
        echo "${ret}"
        return 1
    fi
    # issue #3579: echo the response exactly and remove beginning and trailing quotes only
    echo -e -n "${ret}" | sed -e 's/\\n"$//' -e 's/^"//' -e 's/\\"/"/g'
}

__remote_intern_print_args() {
    __remote_print "Will execute ${cmd}"
    __remote_print "$(echo "Arguments: '${args}'" | sed 's/;/ /g')"
}

__remote_intern_do_cmd() {
    __remote_var_reset
    separator=' '
    __remote_intern_parse_args $@ && \
    __remote_intern_print_args && \
    __remote_intern_exec_cmd
    __remote_var_reset
}

# oload functions

__remote_intern_oload_check_simple_option() {
    tmp=$(echo ${arg} | sed 's/"//g')
    echo -n "${tmp}" | grep -E -- "^-[dfhlqvwAEFORSW]+" > /dev/null 2>&1 && return 0
    echo -n "${tmp}" | grep -E -- "^--(allow-redef|force|full-release|help)" > /dev/null 2>&1 && return 0
    echo -n "${tmp}" | grep -E -- "^--(no-error-update|override|quiet|reload)" > /dev/null 2>&1 && return 0
    echo -n "${tmp}" | grep -E -- "^--(show-(defines|full-error|url))" > /dev/null 2>&1 && return 0
    echo -n "${tmp}" | grep -E -- "^--(use-schedule|verbose|validate)" > /dev/null 2>&1 && return 0
    echo -n "${tmp}" | grep -E -- "^--(import-cfg-val-rest|sign-schema)" > /dev/null 2>&1 && return 0
    return 1
}

__remote_intern_oload_check_long_param_option() {
    tmp=$(echo ${arg} | sed 's/"//g')
    echo -n "${tmp}" | grep -E -- "^--(user-)?schema=" > /dev/null 2>&1 && return 0
    echo -n "${tmp}" | grep -E -- "^--(proxy-)?url=" > /dev/null 2>&1 && return 0
    echo -n "${tmp}" | grep -E -- "^--(data|index)-ts=" > /dev/null 2>&1 && return 0
    echo -n "${tmp}" | grep -E -- "^--delete(-id)?=" > /dev/null 2>&1 && return 0
    echo -n "${tmp}" | grep -E -- "^--(datasource|list|refresh|token)=" > /dev/null 2>&1 && return 0
    echo -n "${tmp}" | grep -E -- "^--(export-cfg-val|show-release)" > /dev/null 2>&1 && return 0
    return 1
}

__remote_intern_oload_check_short_param_option() {
    tmp=$(echo ${arg} | sed 's/"//g')
    echo -n "$tmp" | grep -E -- "^-[prstuDLX]$" > /dev/null 2>&1 && return 0
    return 1
}

__remote_intern_oload_check_short_param_option_with_arg() {
    tmp=$(echo ${arg} | sed 's/"//g')
    echo -n "$tmp" | grep -E -- "^-[prstuDLX]" > /dev/null 2>&1 && return 0
    return 1
}

__remote_intern_oload_parse_args() {
    IFS=';' read -r -a argsarray <<< "${args}"

    oload_files=()
    for arg in "${argsarray[@]}"; do
        # parameter of short option
        if [ "$next_arg_append_opts" = "1" ]; then
            if [ -z "${oload_opts}" ]; then
                oload_opts="${arg}"
            else
                oload_opts="${oload_opts} ${arg}"
            fi
            next_arg_append_opts=
            continue
        fi

        # if simple option, add to args and skip
        if __remote_intern_oload_check_simple_option; then
            if [ -z "${oload_opts}" ]; then
                oload_opts="${arg}"
            else
                oload_opts="${oload_opts} ${arg}"
            fi
            continue
        fi

        # if long param option, add to args and skip
        if __remote_intern_oload_check_long_param_option; then
            if [ -z "${oload_opts}" ]; then
                oload_opts="${arg}"
            else
                oload_opts="${oload_opts} ${arg}"
            fi
            continue
        fi

        # if short param option + next arg param, add to oload opts and set helper var for the next arg
        if __remote_intern_oload_check_short_param_option; then
            if [ -z "${oload_opts}" ]; then
                oload_opts="${arg}"
            else
                oload_opts="${oload_opts} ${arg}"
            fi
            next_arg_append_opts=1
            continue
        fi

        if __remote_intern_oload_check_short_param_option_with_arg; then
            if [ -z "${oload_opts}" ]; then
                oload_opts="${arg}"
            else
                oload_opts="${oload_opts} ${arg}"
            fi
            continue
        fi

        # else file -> add to list
        oload_files+=("${arg}")
    done
}

__remote_intern_oload_check_yaml_files() {
    for ofile in "${oload_files[@]}"; do
        if echo -n ${ofile} | grep -E "\.y[a]?ml" > /dev/null 2>&1; then
            srcfile=`echo -n $ofile | sed 's/\.yml//' | sed 's/\.yaml//'`
            tmp=$(echo ${ofile} | sed 's/"//g')
            if [ -f "${tmp}" ]; then
                oload_files+=("${srcfile}")
            fi
        fi
    done
}

__remote_intern_oload_check_resource_files() {
    resource_files=
    for ofile in "${oload_files[@]}"; do
        tmp=$(echo ${ofile} | sed 's/"//g')
        file_dir=$(dirname "${tmp}")
        __remote_print "Checking resources for file ${ofile}"

        # parse yaml file
        if echo -n "${ofile}" | grep -E "\.y[a]?ml" > /dev/null 2>&1; then
            # skip if the file doesn't contain a resource key
            if ! grep -E "^resource:" "${tmp}" > /dev/null 2>&1; then
                continue
            fi

            # parse the file and it's resource section
            resources=()
            multiline_item_list=
            multiline_bracket_list=
            while IFS= read -r line || [[ -n "$line" ]]; do
                if [ "$multiline_item_list" = "1" ]; then
                    # check if the line is a list item (begins with dash)
                    if echo -n ${line} | grep -E "^[[:blank:]]*-" > /dev/null 2>&1; then
                        rsrc=`echo -n $line | sed 's/^[[:blank:]]*-[[:blank:]]*//' | sed 's/"//g' | xargs`
                        resources+=("${rsrc}")
                        continue
                    fi
                    break
                fi

                if [ "$multiline_bracket_list" = "1" ]; then
                    # check if there is a closing bracket in the line
                    if echo -n ${line} | grep -E "\]" > /dev/null 2>&1; then
                        # parse until closing bracket
                        rsrc=`echo -n $line | sed 's/[[:blank:]]*\(.*\)\].*/\1/g' | sed 's/,/ /g' | sed 's/"//g' | xargs`
                        resources+=("${rsrc}")
                        break
                    fi

                    # parse line
                    rsrc=`echo -n $line | sed 's/,/ /g' | sed 's/"//g' | xargs`
                    resources+=("${rsrc}")

                elif echo -n ${line} | grep -E "^resource:[[:blank:]]*\[" > /dev/null 2>&1; then
                    # parse the rest of the line after opening bracket
                    rsrc=`echo -n $line | sed 's/^resource:[[:blank:]]*\[//'`

                    # check if there is a closing bracket in the line
                    if echo -n ${rsrc} | grep -E "\]" > /dev/null 2>&1; then
                        # parse until closing bracket
                        rsrc=`echo -n $rsrc | sed 's/\].*//' | sed 's/,/ /g' | sed 's/"//g' | xargs`
                        resources+=("${rsrc}")
                        break
                    fi

                    # we are in the middle of multi-line bracket list
                    multiline_bracket_list=1

                    # parse the resource string from the previously parsed line
                    rsrc=`echo -n $rsrc | sed 's/,/ /g' | sed 's/"//g' | xargs`
                    resources+=("${rsrc}")

                elif echo -n ${line} | grep -E "^resource:[[:blank:]]*$" > /dev/null 2>&1; then
                    # this is a beginning of multiline list (dashes in front of items)
                    multiline_item_list=1
                fi
            done < "${tmp}"

        # parse old-style qsd file
        elif echo -n "${tmp}" | grep -E "\.qsd" > /dev/null 2>&1; then
            # skip if the file doesn't contain resource metadata option
            if ! grep -E "#[ ]*resource:" "${tmp}" > /dev/null 2>&1; then
                continue
            fi

            # get resource strings from the metadata options
            resources=`sed 's/#[ ]*resource:[ ]*/XXXOLOAD_RSRC:/' "${tmp}" | grep "XXXOLOAD_RSRC:" | xargs | sed 's/XXXOLOAD_RSRC://g'`

        # non-yaml and non-service files
        else
            continue
        fi

        # get full relative paths of the resources
        resources_full=
        for rsrc in "${resources[@]}"; do
            if [ -z "${resources_full}" ]; then
                resources_full="\"${file_dir}/${rsrc}\""
            else
                resources_full="${resources_full};\"${file_dir}/${rsrc}\""
            fi
        done
        if [ -z "$resources_full" ]; then
            __remote_print "Found resources: ${resources}"
        fi

        # add to resource files list
        if [ -z "${resource_files}" ]; then
            resource_files="${resources_full}"
        else
            resource_files="${resource_files};${resources_full}"
        fi
    done
}

__remote_intern_oload_remove_dir() {
    tmp_files=()
    for ofile in "${oload_files[@]}"; do
        tmp=$(echo ${ofile} | sed 's/"//g')
        if [ ! -d "${tmp}" ]; then
            tmp_files+=("${ofile}")
        fi
    done
    oload_files=("${tmp_files[@]}")
}

__remote_intern_oload_remove_files() {
    tmp_files=()
    for ofile in "${oload_files[@]}"; do
        tmp=$(echo ${ofile} | sed 's/"//g')
        if [ -e "${tmp}" ]; then
            tmp_files+=("${ofile}")
        else
            echo "File does not exist: ${tmp}"
        fi
    done
    oload_files=("${tmp_files[@]}")
}

remote-oload() {
    __remote_var_reset
    cmd=oload
    separator=';'

    __remote_intern_parse_args "$@" || return 1
    __remote_intern_print_args || return 1

    __remote_print "Parsing args for options and loaded files"
    __remote_intern_oload_parse_args ${args} || return 1

    __remote_print "Removing directories from the file list"
    __remote_intern_oload_remove_dir

    __remote_print "Removing non-existing files from the file list"
    __remote_intern_oload_remove_files

    # if there is nothing left in the file list and no opts, stop the script
    if [ -z "${oload_files}" ] && [ -z "${oload_opts}" ]; then
        echo -e "nothing to do !"
        return 1
    fi

    __remote_print "Checking for yaml files and add their source files if they are missing"
    __remote_intern_oload_check_yaml_files

    __remote_print "Checking service files for resources"
    __remote_intern_oload_check_resource_files

    echo -n "Uploading files to remote host \"${URL}\": "
    for ofile in "${oload_files[@]}"; do
        ofile_noquote=$(echo ${ofile} | sed 's/"//g')
        if [ -z ${dir} ]; then
            __remote_print "curl -k -sS ${timeout} --netrc-file ${netrc_file} ${URL}raw/remote-file -H Content-Type:application/octet-stream -H filepath:${ofile_noquote} --data-binary @${ofile_noquote}"
            res=$(curl -k -sS ${timeout} --netrc-file ${netrc_file} ${URL}raw/remote-file -H Content-Type:application/octet-stream -H "filepath:${ofile_noquote}" --data-binary "@${ofile_noquote}")
            if [[ $? > 0 ]] || [[ "${res}" =~ "~<html><head><title>" ]]; then
                echo "${res}"
                return 1
            else
                dir=$(echo ${res} | sed 's/"//g')
                __remote_print "Uploading into ${dir} directory"
            fi
        else
            __remote_print "curl -k -sS ${timeout} --netrc-file ${netrc_file} ${URL}raw/remote-file -H Content-Type:application/octet-stream -H dir:${dir} -H filepath:${ofile_noquote} --data-binary @${ofile_noquote}"
            res=$(curl -k -sS ${timeout} --netrc-file ${netrc_file} ${URL}raw/remote-file -H Content-Type:application/octet-stream -H "dir:${dir}" -H "filepath:${ofile_noquote}" --data-binary "@${ofile_noquote}")
            if [[ $? > 0 ]] || [[ "${res}" =~ "^<html><head><title>" ]]; then
                echo "${res}"
                return 1
            fi
        fi
        echo -n .
        __remote_print "Uploaded ${ofile}"

        if [ -z "${oload_files_final}" ]; then
            oload_files_final="${ofile}"
        else
            oload_files_final="${oload_files_final} ${ofile}"
        fi
    done
    echo

    __remote_print "Uploading resource files"
    IFS=';' read -r -a r_filesarray <<< "${resource_files}"
    for r_file in "${r_filesarray[@]}"; do
        r_file_noquote=$(echo "${r_file}" | sed 's/"//g')
        __remote_print "curl -k -sS ${timeout} --netrc-file ${netrc_file} ${URL}raw/remote-file -H Content-Type:application/octet-stream -H dir:${dir} -H filepath:${r_file_noquote} --data-binary @${r_file_noquote}"
        res=$(curl -k -sS ${timeout} --netrc-file ${netrc_file} ${URL}raw/remote-file -H Content-Type:application/octet-stream -H "dir:${dir}" -H "filepath:${r_file_noquote}" --data-binary "@${r_file_noquote}")
        if [[ $? > 0 ]] || [[ "${res}" =~ "^<html><head><title>" ]]; then
            echo "${res}"
            return 1
        fi
        __remote_print "Uploaded ${r_file}"
    done

    __remote_print "Executing 'oload ${oload_opts} "${oload_files_final}"' on remote host"
    __remote_print "curl -k -sS ${timeout} --netrc-file ${netrc_file} ${URL}api/latest/command -d cmd=${cmd}&token=yes&dir=${dir}&opts=${oload_opts}&files=${oload_files_final}"
    res=$(curl -k -sS ${timeout} --netrc-file ${netrc_file} ${URL}api/latest/command -d "cmd=${cmd}&token=yes&dir=${dir}&opts=${oload_opts}&files=${oload_files_final}")
    if [[ $? > 0 ]] || [[ "${res}" =~ "^<html><head><title>" ]]; then
        echo "${res}"
        return 1
    fi
    # issue #3579: echo the response exactly and remove beginning and trailing quotes only
    echo -e -n "${res}" | sed -e 's/\\n"$//' -e 's/^"//' -e 's/\\"/"/g'

    __remote_print "Deleting folder: ${dir}"
    __remote_print "curl -k -sS -X DELETE --netrc-file ${netrc_file} ${URL}raw/remote-file -H dir:${dir}"
    ret=$(curl -k -sS -X DELETE --netrc-file ${netrc_file} ${URL}raw/remote-file -H "dir:${dir}")
    if [[ $? > 0 ]] || [[ "${res}" =~ "^<html><head><title>" ]]; then
        echo "${res}"
        return 1
    fi
    __reset_token
    __remote_var_reset
}

# make-release functions

__remote_intern_make_release_check_simple_option() {
    tmp=$(echo ${arg} | sed 's/"//g')
    echo -n "$tmp" | grep -E -- "^-[cfhimvCFRS]+" > /dev/null 2>&1 && return 0
    echo -n "$tmp" | grep -E -- "^--(help|compress|refresh(-compat)?)" > /dev/null 2>&1 && return 0
    echo -n "$tmp" | grep -E -- "^--(full-release|install|keep)" > /dev/null 2>&1 && return 0
    echo -n "$tmp" | grep -E -- "^--(fix-module-paths|show-release-dir)" > /dev/null 2>&1 && return 0
    echo -n "$tmp" | grep -E -- "^--(system|verbose|qorusdir)" > /dev/null 2>&1 && return 0
    return 1
}

__remote_intern_make_release_check_long_param_option() {
    tmp=$(echo ${arg} | sed 's/"//g')
    echo -n "$tmp" | grep -E -- "^--(add-)?prefix=" > /dev/null 2>&1 && return 0
    echo -n "$tmp" | grep -E -- "^--load-file=" > /dev/null 2>&1 && return 0
    echo -n "$tmp" | grep -E -- "^--loader-only=" > /dev/null 2>&1 && return 0
    echo -n "$tmp" | grep -E -- "^--user-sql=" > /dev/null 2>&1 && return 0
    echo -n "$tmp" | grep -E -- "^--(release|system)-dir=" > /dev/null 2>&1 && return 0
    return 1
}

__remote_intern_make_release_check_short_param_option() {
    tmp=$(echo ${arg} | sed 's/"//g')
    echo -n "$tmp" | grep -E -- "^-[lpqrsuLP]$" > /dev/null 2>&1 && return 0
    return 1
}

__remote_intern_make_release_check_short_param_option_with_arg() {
    tmp=$(echo ${arg} | sed 's/"//g')
    echo -n "$tmp" | grep -E -- "^-[lpqrsuLP]" > /dev/null 2>&1 && return 0
    return 1
}

__remote_intern_make_release_check_src_dir_option() {
    tmp=$(echo ${arg} | sed 's/"//g')
    echo -n "$tmp" | grep -E -- "^-U(=)?" > /dev/null 2>&1 && return 0
    echo -n "$tmp" | grep -E -- "^--user-src(=)?" > /dev/null 2>&1 && return 0
    return 1
}

__remote_intern_make_release_check_src_dir_option_with_arg() {
    tmp=$(echo "${arg}" | sed 's/"//g')
    echo -n "$tmp" | grep -E -- "^-U$" > /dev/null 2>&1 && return 0
    return 1
}

__remote_intern_make_release_parse_args() {
    IFS=';' read -r -a argsarray <<< "${args}"

    make_release_files=()
    for arg in "${argsarray[@]}"; do
        echo "${arg}"
        # parameter of short option
        if [ "$next_arg_append_opts" = "1" ]; then
            if [ -z "${make_release_opts}" ]; then
                make_release_opts="${arg}"
            else
                make_release_opts="${make_release_opts} ${arg}"
            fi
            next_arg_append_opts=
            continue
        fi

        # parameter of src dir option
        if [ "$next_arg_append_opts" = "2" ]; then
            if [ -z "${make_release_opts}" ]; then
                make_release_opts="${arg}"
            else
                make_release_opts="${make_release_opts} -U${arg}"
            fi
            make_release_src_dir=$(echo "${arg}" | sed 's/-U//g')
            next_arg_append_opts=
            continue
        fi

        # if simple option, add to args and skip
        if __remote_intern_make_release_check_simple_option; then
            if [ -z "${make_release_opts}" ]; then
                make_release_opts="${arg}"
            else
                make_release_opts="${make_release_opts} ${arg}"
            fi
            continue
        fi

        # if long param option, add to args and skip
        if __remote_intern_make_release_check_long_param_option; then
            if [ -z "${make_release_opts}" ]; then
                make_release_opts="${arg}"
            else
                make_release_opts="${make_release_opts} ${arg}"
            fi
            continue
        fi

        # if short param option + next arg param, add to make_release opts and set helper var for the next arg
        if __remote_intern_make_release_check_short_param_option; then
            if [ -z "${make_release_opts}" ]; then
                make_release_opts="${arg}"
            else
                make_release_opts="${make_release_opts} ${arg}"
            fi
            next_arg_append_opts=1
            continue
        fi

        # if short param option with arg, add to args and skip
        if __remote_intern_make_release_check_short_param_option_with_arg; then
            if [ -z "${make_release_opts}" ]; then
                make_release_opts="${arg}"
            else
                make_release_opts="${make_release_opts} ${arg}"
            fi
            continue
        fi

        # if src dir option next arg param, add to make_release opts and set helper var for the next arg
        if __remote_intern_make_release_check_src_dir_option_with_arg; then
            next_arg_append_opts=2
            continue
        fi

        # if src dir option with arg, add to args, extract arg and skip
        if __remote_intern_make_release_check_src_dir_option; then
            if [ -z "${make_release_opts}" ]; then
                make_release_opts="${arg}"
            else
                make_release_opts="${make_release_opts} \"${tmp}\""
            fi
            tmp=$(echo "${tmp}" | sed 's/-U=//g')
            tmp=$(echo "${tmp}" | sed 's/-U//g')
            tmp=$(echo "${tmp}" | sed 's/--user-src=//g')
            tmp=$(echo "${tmp}" | sed 's/--user-src//g')
            make_release_src_dir="${tmp}"
            continue
        fi

        # else file -> add to list
        make_release_files+=("${arg}")
    done

    __remote_print "add / at the end of the src_dir if it is there"
    if [ -z ${make_release_src_dir} ]; then
        make_release_src_dir="./"
    elif [ ! ${make_release_src_dir: -1} == "/" ]; then
        make_release_src_dir=$(echo "${make_release_src_dir}"/ | sed 's/"//g')
    fi

    __remote_print "expand wildcards"
    file_list=()
    for mfile in "${make_release_files[@]}"; do
        mfile_path_noquote=$(echo ${mfile} | sed 's/"//g')
        test=$(eval echo "${make_release_src_dir}""$(dirname "${mfile_path_noquote}")"/$(basename "${mfile_path_noquote}"))
        file_list+=("${test}")
    done

    __remote_print "splitting the label into another variable if it present"
    potential_label=$(echo ${file_list[0]} | sed 's/"//g')
    if [ ! -f "${potential_label}" ]; then
        make_release_label=$(echo "${potential_label}" | sed "s+${make_release_src_dir}++g")
        make_release_label=$(echo "${make_release_label}" | sed "s+./++g")
    fi

    __remote_print "removing the label from the list of files"
    if [ -z ${make_release_label} ]; then
        i=0
    else
        i=1
    fi

    __remote_print "splitting file_list into array"
    make_release_files_inter=()
    for ((; i<${#file_list[@]}; i++ )); do
        s=${file_list[$i]}"${make_release_src_dir}"
        while [[ $s ]]; do
            newfile="$(echo -e "${s%%"${make_release_src_dir}"*}" | sed -e 's/[[:space:]]*$//')"
            if [ ! -z "${newfile}" ]; then
                make_release_files_inter+=("${newfile}")
            fi
            s=${s#*"${make_release_src_dir}"}
        done;
    done

}

remote-make-release() {
    __remote_var_reset
    cmd=make-release
    separator=';'

    __remote_intern_parse_args "$@" || return 1
    __remote_intern_print_args || return 1

    __remote_print "Parsing args for options and loaded files"
    __remote_intern_make_release_parse_args ${args} || return 1

    echo "Uploading files to remote host \"${URL}\": "
    for mfile_inter in "${make_release_files_inter[@]}"; do
        mfile_path=$(echo "${make_release_src_dir}""${mfile_inter}"| sed 's/"//g')
        if [ -z "${dir}" ]; then
            __remote_print "curl -k -sS ${timeout} --netrc-file ${netrc_file} ${URL}raw/remote-file -H Content-Type:application/octet-stream -H filepath:${mfile_path} --data-binary @${mfile_path}"
            res=$(curl -k -sS ${timeout} --netrc-file ${netrc_file} ${URL}raw/remote-file -H Content-Type:application/octet-stream -H "filepath:${mfile_path}" --data-binary "@${mfile_path}")
            if [[ $? > 0 ]] || [[ "${res}" =~ "^<html><head><title>" ]]; then
                echo "${res}"
                return 1
            else
                dir=$(echo ${res} | sed 's/"//g')
                __remote_print "Uploading into ${dir} directory"
            fi
        else
            __remote_print "curl -k -sS ${timeout} --netrc-file ${netrc_file} ${URL}raw/remote-file -H Content-Type:application/octet-stream -H dir:${dir} -H filepath:${mfile_path} --data-binary @${mfile_path}"
            res=$(curl -k -sS ${timeout} --netrc-file ${netrc_file} ${URL}raw/remote-file -H Content-Type:application/octet-stream -H "dir:${dir}" -H "filepath:${mfile_path}" --data-binary "@${mfile_path}")
            if [[ $? > 0 ]] || [[ "${res}" =~ "^<html><head><title>" ]]; then
                echo "${res}"
                return 1
            fi
        fi
        echo -n .
        __remote_print "Uploaded \"${mfile_inter}\""
        if [ -z "${make_release_files_final}" ]; then
            make_release_files_final="\"${mfile_inter}\""
        else
            make_release_files_final="${make_release_files_final} \"${mfile_inter}\""
        fi
    done

    __remote_print "Executing 'make-release ${make_release_opts} ${make_release_label} "${make_release_files_final}"' on remote host"
    __remote_print "curl -k -sS ${timeout} --netrc-file ${netrc_file} ${URL}api/latest/command -d cmd=${cmd}&dir=${dir}&args=${make_release_opts} ${make_release_label} ${make_release_files_final}"
    res=$(curl -k -sS ${timeout} --netrc-file ${netrc_file} ${URL}api/latest/command -d "cmd=${cmd}&dir=${dir}&args=${make_release_opts} ${make_release_label} ${make_release_files_final}")
    if [[ $? > 0 ]] || [[ "${res}" =~ "^<html><head><title>" ]]; then
        echo "${res}"
        return 1
    fi
    __remote_print "${res}"
    res=$(echo "${res}" | sed 's/.*created user tar file //g')
    res=$(echo "${res}" | sed 's/\\n.*//g')
    res=$(echo "${res}" | sed 's/.gz/.bz2/g')
    res=$(basename "${res}")

    echo "Dowloading tar: "${res}""
    __remote_print "curl -k -sS -X GET --netrc-file ${netrc_file} ${URL}raw/remote-file -H dir:${dir} -H filename:${res} --output ${res}"
    ret=$(curl -k -sS -X GET --netrc-file ${netrc_file} ${URL}raw/remote-file -H "dir:${dir}" -H "filename:${res}" --output ${res})
    if [[ $? > 0 ]] || [[ "${ret}" =~ "^<html><head><title>" ]]; then
        echo "${ret}"
        return 1
    fi

    __remote_print "Deleted folder: ${dir}"
    __remote_print "curl -k -sS -X DELETE --netrc-file ${netrc_file} ${URL}raw/remote-file -H dir:${dir}"
    ret=$(curl -k -sS -X DELETE --netrc-file ${netrc_file} ${URL}raw/remote-file -H "dir:${dir}")
    if [[ $? > 0 ]] || [[ "${ret}" =~ "^<html><head><title>" ]]; then
        echo "${ret}"
        return 1
    fi
    __reset_token
    __remote_var_reset
}

remote-ocmd() {
    cmd=ocmd
    has_token="&token=yes"
    __remote_intern_do_cmd $@
}

remote-ojview() {
    cmd=ojview
    has_token="&token=yes"
    __remote_intern_do_cmd $@
}

remote-oprop() {
    cmd=oprop
    has_token="&token=yes"
    __remote_intern_do_cmd $@
}

remote-ostart() {
    cmd=ostart
    has_token="&token=yes"
    __remote_intern_do_cmd $@
}

remote-ostatus() {
    cmd=ostatus
    has_token="&token=yes"
    __remote_intern_do_cmd $@
}

remote-ostop() {
    cmd=ostop
    has_token="&token=yes"
    __remote_intern_do_cmd $@
}

remote-oview() {
    cmd=oview
    has_token="&token=yes"
    __remote_intern_do_cmd $@
}

remote-qctl() {
    cmd=qctl
    __remote_intern_do_cmd $@
}

remote-qrest() {
    cmd=qrest
    __remote_intern_do_cmd $@
}

remote-schema-tool() {
    cmd=schema-tool
    __remote_intern_do_cmd $@
}

remote-user-tool() {
    cmd=user-tool
    __remote_intern_do_cmd $@
}

# qore commands
remote-rest() {
    cmd=rest
    __remote_intern_do_cmd $@
}

remote-schema-reverse() {
    cmd=schema-reverse
    __remote_intern_do_cmd $@
}

remote-sfrest() {
    cmd=sfrest
    __remote_intern_do_cmd $@
}

remote-soaputil() {
    cmd=soaputil
    __remote_intern_do_cmd $@
}

remote-sqlutil() {
    cmd=sqlutil
    __remote_intern_do_cmd $@
}

remote-qdp() {
    cmd=qdp
    __remote_intern_do_cmd $@
}

remote-saprest() {
    cmd=saprest
    __remote_intern_do_cmd $@
}